﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

using org.apache.pdfbox.pdmodel;
using s2industries.ZUGFeRD;

using Mustang;
using System.Windows.Forms;

namespace Zaumzeug
{
    class Zaumzeug
    {
        public Exception CreateSampleZugferdXML(out byte[] zugferdData)
        {
            try
            {
                InvoiceDescriptor desc = InvoiceDescriptor.CreateInvoice("471102", new DateTime(2013, 6, 5), CurrencyCodes.EUR, "GE2020211-471102");
            
                desc.Profile = Profile.Comfort;
                desc.ReferenceOrderNo = "AB-312";
                desc.AddNote("Rechnung gemäß Bestellung Nr. 2013-471331 vom 01.03.2013.");
                desc.AddNote("Es bestehen Rabatt- und Bonusvereinbarungen.", SubjectCode.Unknown);
                desc.SetBuyer("Kunden Mitte AG", "69876", "Frankfurt", "Kundenstraße", "15", "DE", "88", "4000001987658");
                desc.AddBuyerTaxRegistration("DE234567890", TaxRegistrationSchemeID.VA);
                desc.SetBuyerContact("Hans Muster");
                desc.SetSeller("Lieferant GmbH", "80333", "München", "Lieferantenstraße", "20", "DE", "88", "4000001123452");
                desc.AddSellerTaxRegistration("201/113/40209", TaxRegistrationSchemeID.FC);
                desc.AddSellerTaxRegistration("DE123456789", TaxRegistrationSchemeID.VA);
                desc.SetBuyerOrderReferenceDocument("2013-471331", new DateTime(2013, 03, 01));
                desc.SetDeliveryNoteReferenceDocument("2013-51111", new DateTime(2013, 6, 3));
                desc.ActualDeliveryDate = new DateTime(2013, 6, 3);
                desc.SetTotals(202.76m, 5.80m, 14.73m, 193.83m, 21.31m, 215.14m, 50.0m, 165.14m);
                desc.AddApplicableTradeTax(9.06m, 129.37m, 7m, TaxType.VAT, TaxCategoryCode.S);
                desc.AddApplicableTradeTax(12.25m, 64.46m, 19m, TaxType.VAT, TaxCategoryCode.S);
                desc.AddLogisticsServiceCharge(5.80m, "Versandkosten", TaxType.VAT, TaxCategoryCode.S, 7m);
                desc.SetTradePaymentTerms("Zahlbar innerhalb 30 Tagen netto bis 04.07.2013, 3% Skonto innerhalb 10 Tagen bis 15.06.2013", new DateTime(2013, 07, 04));

                String pathZugferdFile = Path.Combine(Path.GetTempPath(), "zugferd.xml");
                desc.Save(pathZugferdFile);

                zugferdData = File.ReadAllBytes(pathZugferdFile);

                try { File.Delete(pathZugferdFile); }
                catch (Exception) { };

                //return ok
                return null;
            }
            catch(Exception ex) 
            { 
                zugferdData = null;
                return ex;
            }
        }

        public Exception attachZUGFeRD(string filepath, byte[] zugferdData)
        {
            try
            {
                // automatically add Zugferd to all outgoing invoices
                ZUGFeRDExporter ze = new ZUGFeRDExporter();
                ze.PDFmakeA3compliant(filepath, "Zaumzeug", Environment.UserName, true);
                ze.PDFattachZugferdFile(zugferdData);
                ze.export(filepath);

                //return ok
                return null;
            }
            catch (Exception ex)
            {
                return ex;
            }
        }

        public Exception showZUGFeRD(string filepath)
        {
            try
            {
                PDDocument doc = PDDocument.load(filepath);

                // now check the contents (like MustangReaderTest)
                ZUGFeRDImporter zi = new ZUGFeRDImporter();
                zi.extract(filepath);

                // Reading ZUGFeRD
                if (zi.canParse())
                {
                    zi.parse();

                    //show some of the extracted ZUGFeRD data
                    MessageBox.Show(
                        String.Format("Amount: {0}\nHolder: {1}\nReference: {2}", 
                            zi.getAmount(), zi.getHolder(), zi.getForeignReference()));
                }
                else
                {
                    MessageBox.Show("no ZUGFeRD data found");
                }

                //return ok
                return null;
            }
            catch (Exception ex)
            {
                return ex;
            }
        }
    }
}
